#include <Wire.h>
#include <EEPROM.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 20, 4);  //вместо (0x27) указываем адрес своего дисплея //закомментировать если дисплей без I2C

//---секция ПИД---//
#define PID_SAMPLING_TIME 100
double  Input1; //текущая температура(измеренная max6675) ВИ
byte Output1; //выход ПИД регулятора ВИ
byte Setpoint1; //установленная температура(уставка) ВИ
double  Input2; //текущая температура(измеренная max6675) НИ
byte Output2; //установленная температура(уставка) НИ
int Setpoint2; //выход ПИД регулятора НИ
byte kp1; //пропорциональный коэффициент ПИД для ВИ
byte ki1; //интегральный коэффициент ПИД для ВИ
byte kd1; //дифференциальный коэффициент ПИД для ВИ
byte kp2; //пропорциональный коэффициент ПИД для НИ
byte ki2; //интегральный коэффициент ПИД для НИ
byte kd2; //дифференциальный коэффициент ПИД для НИ
#define i_min 0.0//минимум И составляющей
#define i_max 100.0//максимум И составляющей
float integra, integra2;
//---секция ПИД---//

//---MAX6675 functions---//
double max6675_read_temp (int CLK, int CS, int SO)
{ char i;
  int tmp = 0;
  digitalWrite(CS, LOW); //CS = 0;                            //Stop a conversion in progress
  asm volatile
  (
    " nop"        "\n\t"
  );
  for (i = 15; i >= 0; i--)
  { digitalWrite(CLK, HIGH);
    asm volatile
    (
      " nop"        "\n\t"
    );
    if ( digitalRead(SO))
      tmp |= (1 << i);


    digitalWrite(CLK, LOW);
    asm volatile
    (
      " nop"        "\n\t"
    );
  }
  digitalWrite(CS, HIGH);
  if (tmp & 0x4) {
    return NAN;
  } else
    return ((tmp >> 3)) * 0.25;
}
//---MAX6675 functions---//

//---секция алгоритма Брезенхема---//
int er1 = 1;  //переменная для расчета переполнения в алгоритме Брезенхема для ВИ
int er2 = 1;  //переменная для расчета переполнения в алгоритме Брезенхема для НИ
int reg1;     //промежуточная переменная для алгоритма Брезенхема для ВИ
int reg2;     //промежуточная переменная для алгоритма Брезенхема для НИ
boolean out1; //выход импульса управления ВИ алгоритма Брезенхема
boolean out2; //выход импульса управления НИ алгоритма Брезенхема
//---секция алгоритма Брезенхема---//

//---секция нагревателей---//
//переменные для установки мощности нагревателей
byte pwr_TOP;     //максимальная мощность верхнего нагревателя
byte pwr_BOTTOM;  //максимальная мощность нижнего нагревателя
//RelayPin "1"-ВЕРХНИЙ нагреватель
//RelayPin "2"-НИЖНИЙ нагреватель
#define RelayPin1 12  //назначаем пин "ВЕРХНЕГО" нагревателя
#define RelayPin2 11  //назначаем пин "НИЖНЕГО" нагревателя
//---секция нагревателей---//

//---секция кнопок---//
//Назначаем пины кнопок управления
byte upSwitchPin = A0;
byte downSwitchPin = A1;
byte cancelSwitchPin = A2;
byte okSwitchPin = A3;
//состояние кнопок по умолчанию
boolean upSwitchState = 0;
boolean downSwitchState = 0;
boolean cancelSwitchState = 0;
boolean okSwitchState = 0;
//переменные для кнопок
long ms_button = 0;
boolean  button_state = false;
boolean  button_long_state = false;
//---секция кнопок---//

//---секция профиля---//
byte currentProfile = 1; //текущий профиль
byte currentStep = 1; //текущий шаг профиля
byte profileSteps; //количество шагов профиля
byte editStep = 0; //номер шага профиля который редактируется в данный момент
double rampRateStep[9]; //скорость роста температуры
byte dwellTimerStep[9]; //установленное время перехода на следубщий шаг
double startTemp; //стартовая температура для профиля, когда НИ вышел на уставку, используется для определения шага профиля с которого начинаем
byte setpointRamp; //заданная температура по определённому шагу профиля ВИ, связана с заданной скоростью роста температуры
boolean TopStart = false; //флаг включения ВИ
boolean flag = 0; //флаг для фиксации стартовой температуры и начала работы нагревателей
byte x = 1; //переменная для перехода на нужный шаг при горячей плате
byte temperatureStep[9]; //заданные температуры для шагов профиля ВИ
//---секция профиля---//

//---структура статусов событий---//
// ***** TYPE DEFINITIONS *****
typedef enum REFLOW_STATE : byte
{
  REFLOW_STATE_IDLE,
  REFLOW_STATE_MENU_STEPS,
  REFLOW_STATE_MENU_BOTTOM_HEAT,
  REFLOW_STATE_MENU_BOTTOM_PWR,
  REFLOW_STATE_MENU_STEP_RAMP,
  REFLOW_STATE_MENU_STEP_TARGET,
  REFLOW_STATE_MENU_TOP_PWR,
  REFLOW_STATE_MENU_STEP_DWELL,
  REFLOW_STATE_MENU_BOTTOM_P,
  REFLOW_STATE_MENU_BOTTOM_I,
  REFLOW_STATE_MENU_BOTTOM_D,
  REFLOW_STATE_MENU_TOP_P,
  REFLOW_STATE_MENU_TOP_I,
  REFLOW_STATE_MENU_TOP_D,
  REFLOW_STATE_STEP_RAMP,
  REFLOW_STATE_STEP,
  REFLOW_STATE_STEP_DWELL,
  REFLOW_STATE_COMPLETE,
  REFLOW_STATE_ERROR
}
reflowState_t;

typedef enum REFLOW_STATUS : byte  //структура статуса пайки, запущен или нет
{
  REFLOW_STATUS_OFF,
  REFLOW_STATUS_ON
}
reflowStatus_t;
reflowStatus_t reflowStatus; //переменная статусов событий
reflowState_t reflowState; //переменная статуса пайки
//---структура статусов событий---//

#define SENSOR_SAMPLING_TIME 250 //частота обновления текущей температуры (1000=1 раз в секунду)
#define GRAPHICS_SAMPLING_TIME 500 //скорость движения "звёздочек" при пайке
unsigned long nextRead1; //переменная для обновления текущей температуры
unsigned long nextRead2; //переменная для движения "звёздочек" при пайке
unsigned long nextRead3; //переменная для скорости ПИД
float Input_f1, Input_f2; //это для графиков на ПК
unsigned long windowStartTime;
boolean alarmOn = false; //признак предупреждения
boolean updateScreen = true; //признак обновления экрана
int eepromAddress = 0; //начало eepromaddress
byte Secs, prev_millis = 0; //это для графиков на ПК
byte curCount = 0;
long previousMillis; //это для счетчиков
volatile uint8_t counter;

#define buzzerPin  10 //назначаем пин пищалки
#define Int_Fan  4  //назначаем пин охладителя симисторов и контроллера
#define Ext_Fan  5  //назначаем пин охладителя платы

//назначаем пины преобразователей термопар
#define thermoSO  6 //DO=(SO)
#define thermoCLK  7 //SCK=(CLK)
#define thermoCStop  8 //CS MAX6675 "ВЕРХНЕГО" нагревателя
#define thermoCSbott  9 //CS MAX6675 "НИЖНЕГО" нагревателя

//переменные для калмана "ВЕРХНЕГО" нагревателя
float varTerm1 = 0.25;  //среднее отклонение (ищем в excel)
float varProcess1 = 0.0125; //скорость реакции на изменение (подбирается вручную)
float Pc1 = 0.0;
float G1 = 0.0;
float P1 = 1.0;
float Xp1 = 0.0;
float Zp1 = 0.0;
float Xe1 = 0.0;

//переменные для калмана "НИЖНЕГО" нагревателя
float varTerm2 = 0.25;  //среднее отклонение (ищем в excel)
float varProcess2 = 0.0125; //скорость реакции на изменение (подбирается вручную)
float Pc2 = 0.0;
float G2 = 0.0;
float P2 = 1.0;
float Xp2 = 0.0;
float Zp2 = 0.0;
float Xe2 = 0.0;

//переменные для обоих фильтров калмана
int tc1; //переменная для хранения текущей температуры ВИ
int tc2; //переменная для хранения текущей температуры НИ

void loadProfile() //чтение профиля из EEPROM
{
  profileSteps = EEPROM.read((currentProfile - 1) * 29);
  Setpoint2 = 80 + EEPROM.read((currentProfile - 1) * 29 + 1);
 /*
   for (int i = 0; i < 9; i + 1) {
    rampRateStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 2);
    i++;
  }
  for (int i = 0; i < 9; i + 1) {
    dwellTimerStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 11) * 5;
    i++;
  }
  for (int i = 0; i < 9; i + 1) {
    temperatureStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 20);
    i++; 
    */
  for (int i = 0; i < 9; i++) {
    rampRateStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 2);
    dwellTimerStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 11) * 5;
    temperatureStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 20);
    }
  kp1 = EEPROM.read((currentProfile - 1) * 8 + 122);
  ki1 = EEPROM.read((currentProfile - 1) * 8 + 123);
  kd1 = EEPROM.read((currentProfile - 1) * 8 + 124);
  kp2 = EEPROM.read((currentProfile - 1) * 8 + 125);
  ki2 = EEPROM.read((currentProfile - 1) * 8 + 126);
  kd2 = EEPROM.read((currentProfile - 1) * 8 + 127);
  pwr_TOP = EEPROM.read((currentProfile - 1) * 8 + 128);
  pwr_BOTTOM = EEPROM.read((currentProfile - 1) * 8 + 129);

  return;
}

void setup()
{
  //Записываем единицы в EEPROM для избавления от глюков. Закомментировать после разового запуска
  //for (int i = 0; i < 1024; i++)
  //EEPROM.write(i, 1);

  Serial.begin(9600);

  loadProfile();//вызов функции loadProfile для загрузки данных профиля из eeprom

  //attachInterrupt(0, Dimming, RISING); //настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
  attachInterrupt(0, Dimming, FALLING); //настроить порт прерывания(0 или 1) 2й или 3й цифровой пин
  /* -------------------------------------------------------------------------------
    LOW вызывает прерывание, когда на порту LOW
    CHANGE прерывание вызывается при смене значения на порту, с LOW на HIGH и наоборот
    RISING прерывание вызывается только при смене значения на порту с LOW на HIGH
    FALLING прерывание вызывается только при смене значения на порту с HIGH на LOW
    ------------------------------------------------------------------------------- */

  //устанавливаем пины кнопок на ВХОД и подключаем подтягивающие резисторы
  pinMode (upSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор
  pinMode (downSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор
  pinMode (cancelSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор
  pinMode (okSwitchPin, INPUT_PULLUP); //подключен подтягивающий резистор

  //задаём состояние пинов, к которым подключены MAX6675
  pinMode(thermoCStop, OUTPUT);
  pinMode(thermoCSbott, OUTPUT);
  pinMode(thermoCLK, OUTPUT);
  pinMode(thermoSO, INPUT);

  //устанавливаем пин пищалки на ВЫХОД
  pinMode(buzzerPin, OUTPUT);

  //устанавливаем пины охладителей на ВЫХОД
  pinMode (Int_Fan, OUTPUT);
  pinMode (Ext_Fan, OUTPUT);

  //задаём состояние пинов управления выходами
  pinMode(RelayPin1, OUTPUT);
  pinMode(RelayPin2, OUTPUT);

  //сообщение приветствия
  lcd.begin();
  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print("ARDUINO REWORK");
  lcd.setCursor(7, 1);
  lcd.print("v1.3.3");
  lcd.setCursor(2, 2);
  lcd.print("Dmitrysh edition");
  lcd.setCursor(2, 3);
  lcd.print("max6675  lcd2004");

  //Мелодия приветствия
  tone(buzzerPin, 523);
  delay(200);
  tone(buzzerPin, 659);
  delay(200);
  tone(buzzerPin, 784);
  delay(200);
  tone(buzzerPin, 1046);
  delay(200);
  noTone(buzzerPin);
 
  //показываем заставку пока стабилизируются MAX6675
  delay(5000);
  lcd.clear();


  nextRead1 = millis(); //запускаем чтение с термопары ВИ
  //initialize soak timer variable
}

int i = 0;
char buf[32]; //это для графиков на ПК

void loop()
{
  //Считываем состояние кнопок управления
  upSwitchState = digitalRead(upSwitchPin);
  downSwitchState = digitalRead(downSwitchPin);
  cancelSwitchState = digitalRead(cancelSwitchPin);
  okSwitchState = digitalRead(okSwitchPin);
  //------------------------------------------
  if (upSwitchState == HIGH) {
    upSwitchState = LOW;
  } else {
    upSwitchState = HIGH;

  }
  if (downSwitchState == HIGH) {
    downSwitchState = LOW;
  } else {
    downSwitchState = HIGH;
  }
  if (cancelSwitchState == HIGH) {
    cancelSwitchState = LOW;
  } else {
    cancelSwitchState = HIGH;
  }
  if (okSwitchState == HIGH) {
    okSwitchState = LOW;
  } else {
    okSwitchState = HIGH;
  }
  //----------------------------------------------

  unsigned long currentMillis = millis();

  //int SP1 = Setpoint1;
  int SP2 = Setpoint2;

  //отключил бузер при нажатии на кнопки (нервирует), кому надо раскоментируйте ниже 6 строк
  //  if (upSwitchState == HIGH || downSwitchState == HIGH || cancelSwitchState == HIGH || okSwitchState == HIGH)
  //  {
  //    tone(buzzerPin, 1045);
  //    delay(100);
  //     noTone(buzzerPin);
  //  }

  if (reflowState == REFLOW_STATE_COMPLETE || alarmOn) {
    if (i < 2 && cancelSwitchState == LOW) {
      alarmOn = true;
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      i++;
    }
    else {
      i = 0;
      alarmOn = false;
    }
  }

  switch (reflowState)
  {
    case REFLOW_STATE_IDLE:
      TopStart = false;
      currentStep = 1;
      counter = 0;
      setpointRamp = 0;
      x = 1;             //устанавливаем переменную в исходное состояние
      flag = 0;         //после отановки профиля сбрасываем флаг
      Output1 = 0;
      Output2 = 0;

      if (millis() > nextRead1)
      {
        //Read thermocouples next sampling period
        nextRead1 = millis() + SENSOR_SAMPLING_TIME; //устраняем колебания датчиков температуры
        Input1 = Input_f1 = filter1(max6675_read_temp (thermoCLK, thermoCStop, thermoSO)); //термопара "ВЕРХНЕГО" нагревателя
        Input2 = Input_f2 = filter2(max6675_read_temp (thermoCLK, thermoCSbott, thermoSO)); //термопара "НИЖНЕГО"  нагревателя
        tc1 = Input1;
        tc2 = Input2;

        sprintf(buf, "$%d %d;", int(Input_f2), int(Input_f1)); //график на ПК

        lcd.setCursor(16, 2);
        lcd.print("    ");
        lcd.setCursor(16, 2);
        if (isnan(Input1)) {
          lcd.print("Er");
        } else {
          lcd.print(tc1);
        }
        lcd.setCursor(16, 3);
        lcd.print("    ");
        lcd.setCursor(16, 3);
        if (isnan(Input2)) {
          lcd.print("Er");
        } else {
          lcd.print(tc2);
        }
      }

      //Обновляем экран один раз
      if (updateScreen) {

        
        //Настройка экрана в режиме ожидания
        lcd.clear();
        lcd.setCursor(8, 0);
        lcd.print("IDLE");
        lcd.setCursor(1, 1);
        lcd.print("PTN:");
        lcd.print("      STEP:1 ");
        lcd.setCursor(0, 2);
        lcd.print(" TH  SP:");
        lcd.setCursor(13, 2);
        lcd.print("PV:");
        lcd.setCursor(0, 3);
        lcd.print(" BH  SP:");
        lcd.setCursor(13, 3);
        lcd.print("PV:");
        updateScreen = false;
      }
      lcd.setCursor(5, 1);
      lcd.print(currentProfile);
      lcd.print(" ");
      lcd.setCursor(8, 2);
      lcd.print(temperatureStep[0]);
      lcd.print(" ");
      lcd.setCursor(8, 3);
      lcd.print(SP2);
      lcd.print("  ");
      //windowStartTime = millis();

      if (upSwitchState == HIGH && ( millis() - ms_button) > 500) //if up switch is pressed go to next profile
      {
        ms_button =  millis();
        currentProfile = currentProfile + 1;
        if (currentProfile >= 5) //if currentProfile = 4 and up is pressed go back to profile 1
        {
          currentProfile = 1;
        }
        loadProfile(); //вызов функции loadProfile для загрузки данных профиля из eeprom
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 500) //same as above just go down one profile
      {
        ms_button =  millis();
        currentProfile = currentProfile - 1;
        if (currentProfile <= 0)
        {
          currentProfile = 4;
        }
        loadProfile(); //вызов функции loadProfile для загрузки данных профиля из eeprom
      }

 
      //фиксируем момент нажатия кнопки "ОК" + защита от дребезга
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
        button_long_state = false;
      }

      
      //держим "ОК" 3 секунды и заходим в меню настроек
      if (okSwitchState == HIGH && !button_long_state && ( millis() - ms_button) > 3000)
      {
        button_long_state = true;
        button_state = false;
        reflowState = REFLOW_STATE_MENU_STEPS;
        //update next screen
        updateScreen = true;
      }
 
      
      //включаем пайку, кнопка сработает после отпускания "ОК" если нажатие меньше 3х секунд
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        tone(buzzerPin, 1045, 500);  //звуковой сигнал при старте профиля
        //update next screen
        updateScreen = true;
        curCount = 0;
        nextRead2 = millis();
        nextRead1 = millis();
        nextRead3 = millis();
        reflowStatus = REFLOW_STATUS_ON;
        reflowState = REFLOW_STATE_STEP_RAMP;
      }
      break;
 
    
    //устанавливаем количество шагов профиля
    case REFLOW_STATE_MENU_STEPS:
      if (updateScreen) {
        lcd.clear();
        lcd.setCursor(3, 0);
        lcd.print("Profile ");
        lcd.print(currentProfile);
        lcd.print(" Edit");
        lcd.setCursor(0, 1);
        lcd.print("                    ");
        lcd.setCursor(2, 2);
        lcd.print("Profile Steps: ");
        lcd.print(profileSteps);
        updateScreen = false;
      }
      lcd.setCursor(17, 2);
      lcd.print(profileSteps);

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profileSteps = profileSteps + 1;
        if (profileSteps >= 5) {
          profileSteps = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        profileSteps = profileSteps - 1;
        if (profileSteps <= 0)
        {
          profileSteps = 4;
        }
      }
      if (okSwitchState == LOW && button_long_state) button_long_state = false; //чтобы после входа в меню кнопка "ОК" не считалась нажатой

      if (okSwitchState == HIGH && !button_state && !button_long_state && ( millis() - ms_button) > 100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_HEAT;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
  
    
    //устанавливаем температуру "Нижнего Нагревателя"
    case REFLOW_STATE_MENU_BOTTOM_HEAT:

      if (updateScreen) {
        lcd.setCursor(2, 2);
        lcd.print("Bottom Heat:      ");
        //updateScreen = true;
      }
      lcd.setCursor(14, 2);
      lcd.print(SP2);

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        Setpoint2 = Setpoint2 + 10;
        if (Setpoint2 >= 330)
        {
          Setpoint2 = 330;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        Setpoint2 = Setpoint2 - 10;
        if (Setpoint2 <= 80)
        {
          Setpoint2 = 80;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

 
    //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"
    case REFLOW_STATE_MENU_BOTTOM_PWR:

      if (updateScreen)
      {

        lcd.setCursor(2, 2);
        lcd.print("Bottom PWR:    ");
        updateScreen = false;
      }
      lcd.setCursor(13, 2);
      lcd.print(pwr_BOTTOM);
      lcd.print("%");

      if (pwr_BOTTOM <= 99)
      {
        lcd.setCursor(16, 2);
        lcd.print("  ");
      }
      if (pwr_BOTTOM <= 9)
      {
        lcd.setCursor(15, 2);
        lcd.print("  ");
      }

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        pwr_BOTTOM = pwr_BOTTOM + 1;
        if (pwr_BOTTOM >= 100)
        {
          pwr_BOTTOM = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        pwr_BOTTOM = pwr_BOTTOM - 1;
        if (pwr_BOTTOM <= 0)
        {
          pwr_BOTTOM = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_PWR;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

    
    //устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"
    case REFLOW_STATE_MENU_TOP_PWR:

      if (updateScreen)
      {

        lcd.setCursor(2, 2);
        lcd.print("   TOP PWR:    ");
        updateScreen = false;
      }
      lcd.setCursor(13, 2);
      lcd.print(pwr_TOP);
      lcd.print("%");

      if (pwr_TOP <= 99)
      {
        lcd.setCursor(16, 2);
        lcd.print("  ");
      }
      if (pwr_TOP <= 9)
      {
        lcd.setCursor(15, 2);
        lcd.print("  ");
      }

      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        pwr_TOP = pwr_TOP + 1;
        if (pwr_TOP >= 100)
        {
          pwr_TOP = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        pwr_TOP = pwr_TOP - 1;
        if (pwr_TOP <= 0)
        {
          pwr_TOP = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_STEP_RAMP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;


    //устанавливаем скорость нагрева "Верхним Нагревателем"
    case REFLOW_STATE_MENU_STEP_RAMP:

      if (updateScreen) {
        lcd.setCursor(2, 2);
        lcd.print("Step ");
        lcd.print(editStep + 1);
        lcd.print(" Ramp:      ");
        updateScreen = false;
      }
      lcd.setCursor(14, 2);
      lcd.print(rampRateStep[editStep] / 4);
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        rampRateStep[editStep] = rampRateStep[editStep] + 1;
        if (rampRateStep[editStep] >= 12.)  //(только целые числа) максимальная скорость роста температуры делённая на 4
        {
          rampRateStep[editStep] = 12.;  //(только целые числа) максимальная скорость роста температуры делённая на 4
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        rampRateStep[editStep] = rampRateStep[editStep] - 1;
        if (rampRateStep[editStep] <= 1.)  //(только целые числа) минимальная скорость роста температуры делённая на 4
        {
          rampRateStep[editStep] = 1.;  //(только целые числа) минимальная скорость роста температуры делённая на 4
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_TARGET;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 
    
    //устанавливаем температуру "Верхнего Нагревателя"
    case REFLOW_STATE_MENU_STEP_TARGET:
      if (updateScreen) {
        lcd.setCursor(2, 2);
        lcd.print("Step ");
        lcd.print(editStep + 1);
        lcd.print(" Target: ");
        lcd.print(temperatureStep[editStep]);
        lcd.print("  ");       
        updateScreen = false;
      }
      lcd.print("  ");
      lcd.setCursor(16, 2);
      lcd.print(temperatureStep[editStep]);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        temperatureStep[editStep] = temperatureStep[editStep] + 1;
        if (temperatureStep[editStep] >= 255)
        {
          temperatureStep[editStep] = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        temperatureStep[editStep] = temperatureStep[editStep] - 1;
        if (temperatureStep[editStep] <= 0)
        {
          temperatureStep[editStep] = 0;
        }
        if (temperatureStep[editStep] <= 99)
        {
          lcd.setCursor(18, 2);
          lcd.print("  ");
        }
        if (temperatureStep[editStep] <= 9)
        {
          lcd.setCursor(17, 2);
          lcd.print("  ");
        }
      }

      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_DWELL;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 
    
    //устанавливаем время перехода на следующий шаг
    case REFLOW_STATE_MENU_STEP_DWELL:
      if (updateScreen) {
        lcd.setCursor(1, 2);
        lcd.print("Step ");
        lcd.print(editStep + 1);
        lcd.print(" Dwell: ");
        lcd.print(dwellTimerStep[editStep]);
        lcd.print("  ");
      }
      lcd.print("  ");
      lcd.setCursor(15, 2);
      lcd.print(dwellTimerStep[editStep]);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        dwellTimerStep[editStep] = dwellTimerStep[editStep] + 5;
        if (dwellTimerStep[editStep] >= 100)
        {
          dwellTimerStep[editStep] = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        dwellTimerStep[editStep] = dwellTimerStep[editStep] - 5;
        if (dwellTimerStep[editStep] <= 0)
        {
          dwellTimerStep[editStep] = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {

          editStep = 0;
          reflowState = REFLOW_STATE_MENU_BOTTOM_P;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 
    
    //настройка "ПИД" нижнего нагревателя
    case REFLOW_STATE_MENU_BOTTOM_P:
      if (updateScreen) {
        lcd.setCursor(0, 2);
        lcd.print("                  ");
        lcd.setCursor(3, 2);
        lcd.print("Bottom Heater:  ");
        lcd.setCursor(8, 3);
        lcd.print("P=");
        lcd.print(kp2);
        lcd.print("  ");
        updateScreen = false;
      }
      lcd.setCursor(10, 3);
      lcd.print(kp2);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kp2 = kp2 + 1;
        if (kp2 >= 255)
        {
          kp2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kp2 = kp2 - 1;
        if (kp2 <= 0)
        {
          kp2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_I;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_BOTTOM_I:
      lcd.setCursor(8, 3);
      lcd.print("I=");
      lcd.print(ki2);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        ki2 = ki2 + 1;
        if (ki2 >= 255)
        {
          ki2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        ki2 = ki2 - 1;
        if (ki2 <= 0)
        {
          ki2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_BOTTOM_D;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_BOTTOM_D:
      lcd.setCursor(8, 3);
      lcd.print("D=");
      lcd.print(kd2);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kd2 = kd2 + 1;
        if (kd2 >= 255)
        {
          kd2 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kd2 = kd2 - 1;
        if (kd2 <= 0)
        {
          kd2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_TOP_P;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 
    
    //настройка "ПИД" верхнего нагревателя
    case REFLOW_STATE_MENU_TOP_P:
      if (updateScreen) {
        lcd.setCursor(0, 2);
        lcd.print("                  ");
        lcd.setCursor(5, 2);
        lcd.print("Top Heater:");
        lcd.setCursor(8, 3);
        lcd.print("P=");
        lcd.print(kp1);
        lcd.print("  ");
        updateScreen = false;
      }
      lcd.setCursor(10, 3);
      lcd.print(kp1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kp1 = kp1 + 1;
        if (kp1 >= 255)
        {
          kp1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kp1 = kp1 - 1;
        if (kp1 <= 0)
        {
          kp1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_I;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_I:
      lcd.setCursor(8, 3);
      lcd.print("I=");
      lcd.print(ki1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        ki1 = ki1 + 1;
        if (ki1 >= 255)
        {
          ki1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        ki1 = ki1 - 1;
        if (ki1 <= 0)
        {
          ki1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;
        reflowState = REFLOW_STATE_MENU_TOP_D;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_D:
      lcd.setCursor(8, 3);
      lcd.print("D=");
      lcd.print(kd1);
      lcd.print("  ");
      if (upSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kd1 = kd1 + 1;
        if (kd1 >= 255)
        {
          kd1 = 255;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        kd1 = kd1 - 1;
        if (kd1 <= 0)
        {
          kd1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button) > 200)
      {
        ms_button =  millis();
        button_state = false;

        
        //сохраняем текущие параметры в "Память"
        EEPROM.write((currentProfile - 1) * 29, profileSteps);
        EEPROM.write((currentProfile - 1) * 29 + 1, (Setpoint2 - 80) );
      /*
         for (int i = 0; i < 9; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 2), rampRateStep[i] );
          i++;
        }
        for (int i = 0; i < 9; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 11), dwellTimerStep[i] / 5);
          i++;
        }
        for (int i = 0; i < 9; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 20), temperatureStep[i]);
          i++;
          */
        for (int i = 0; i < 9; i++) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 2), rampRateStep[i] );
          EEPROM.write(((currentProfile - 1) * 29 + i + 11), dwellTimerStep[i] / 5);
          EEPROM.write(((currentProfile - 1) * 29 + i + 20), temperatureStep[i]);
         }
        EEPROM.write(((currentProfile - 1) * 8 + 122), kp1);
        EEPROM.write(((currentProfile - 1) * 8 + 123), ki1);
        EEPROM.write(((currentProfile - 1) * 8 + 124), kd1);
        EEPROM.write(((currentProfile - 1) * 8 + 125), kp2);
        EEPROM.write(((currentProfile - 1) * 8 + 126), ki2);
        EEPROM.write(((currentProfile - 1) * 8 + 127), kd2);

        EEPROM.write(((currentProfile - 1) * 8 + 128), pwr_TOP);
        EEPROM.write(((currentProfile - 1) * 8 + 129), pwr_BOTTOM);


        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        lcd.clear();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 
    
    //"процесс пайки", рост температуры с заданной скоростью
    case REFLOW_STATE_STEP_RAMP:
      digitalWrite(Int_Fan, HIGH); //включить охладитель симисторов и контроллера
      //currentStep = 1;
      if (updateScreen) {
        lcd.setCursor(8, 0);
        lcd.print("RUN!");
        updateScreen = false;
      }

      //if (tc2 >= SP2-2 && !TopStart) TopStart = true;  //если температура низа вышла на уставку, включаем верхний нагреватель
      //if (tc1 >= 118 && !TopStart) TopStart = true; //если температура низа больше или равна 118*С включаем верхний нагреватель    
      //if (tc1 >= SP2 - 2 && !TopStart) TopStart = true; //если температура низа вышла на уставку, включаем верхний нагреватель
if (tc1 >= 29 && !TopStart) TopStart = true;

      if (TopStart == true) //включаем верхний нагреватель
        //-------------------------------------------------------------------------------------------
      { //открываем скобку (чтоб не запутаться)

        //фиксируем стартовую температуру
        if (flag == 0)
        {
          startTemp = tc1;
          flag = 1;
        }
 
        //устанавливаем нужный шаг, до которой нагрета плата
        if (startTemp > temperatureStep[currentStep - 1]) {
          for (x = 1; startTemp > temperatureStep[currentStep - 1]; currentStep++) {
            x++;
          }
        }
        if (currentStep > x && flag == 1) {
          flag = 0;
          startTemp = temperatureStep[currentStep - 2];
          flag = 1;
        }
        lcd.setCursor(16, 1);
        lcd.print(currentStep);
        lcd.setCursor(8, 3);
        lcd.print(SP2);

 
        //счётчик скорости роста температуры
        if ((currentMillis - previousMillis) > 4000 / rampRateStep[currentStep - 1])  //seconds counter
        {
          previousMillis = currentMillis;
          counter = counter + 1;
          setpointRamp = counter + startTemp;
          lcd.setCursor(8, 2);
          lcd.print("   ");
          lcd.setCursor(8, 2);
          lcd.print(setpointRamp);
          lcd.print("  ");
          Setpoint1 = setpointRamp;
        }
      } //закрывам скобку
      //----------------------------------------------------------------------------------------

      if (setpointRamp >= temperatureStep[currentStep - 1]) //если достигли нужной температуры
      {
        lcd.setCursor(8, 2);
        lcd.print(temperatureStep[currentStep - 1]);
        reflowState = REFLOW_STATE_STEP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        digitalWrite(Int_Fan, LOW); //выключить охладитель симисторов и контроллера
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
        updateScreen = true;
      }
      break;
    case REFLOW_STATE_STEP:
      Setpoint1 = temperatureStep[currentStep - 1];
      if (Input1 >= temperatureStep[currentStep - 1])
      {
        counter = 0;
        reflowState = REFLOW_STATE_STEP_DWELL;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;

        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 
    
    //считаем время перехода на следующий шаг
    case REFLOW_STATE_STEP_DWELL:
      if (currentMillis - previousMillis > 1000)
      {
        previousMillis = currentMillis;
        counter = counter + 1;
      }
      if (counter == dwellTimerStep[currentStep - 1]) //если счётчик равен установленному времени
      {
        counter = 0;
        setpointRamp = 0;
        if (profileSteps == currentStep) //если достигли последнего шага
        {
          currentStep = 1;
          x = 1;      //устанавливаем переменную в исходное состояние
          flag = 0;   //после завершения профиля сбрасываем флаг
          reflowState = REFLOW_STATE_COMPLETE;
        }
        else //если шаг не последний
        {
          currentStep++; //переходим на следующий шаг
          reflowState = REFLOW_STATE_STEP_RAMP;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button) > 50)
      {
        ms_button =  millis();
        updateScreen = true;

        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
  
    
    //завершение пайки
    case REFLOW_STATE_COMPLETE:
      digitalWrite(Ext_Fan, HIGH); //включить охладитель платы
      digitalWrite(Int_Fan, LOW); //выключить охладитель симисторов и контроллера
      reflowStatus = REFLOW_STATUS_OFF;
      reflowState = REFLOW_STATE_IDLE;
      updateScreen = true;
      break;
  }
 
  
  //включение нагревателей
  if (reflowStatus == REFLOW_STATUS_ON)
  {
    if (millis() > nextRead2)
    {
      Input1 = Input_f1 = filter1(max6675_read_temp (thermoCLK, thermoCStop, thermoSO)); //термопара "ВЕРХНЕГО" нагревателя
      Input2 = Input_f2 = filter2(max6675_read_temp (thermoCLK, thermoCSbott, thermoSO)); //термопара "НИЖНЕГО"  нагревателя

      tc1 = Input1;
      tc2 = Input2;

      nextRead2 = millis() + GRAPHICS_SAMPLING_TIME;
      if (curCount <= 6)
      {
        lcd.setCursor(curCount, 0);
        lcd.print(" *");
        lcd.setCursor(18 - curCount, 0);
        lcd.print("* ");
      }
      if (curCount > 6)
      {
        lcd.setCursor(13 - curCount, 0);
        lcd.print("* ");
        lcd.setCursor(curCount + 5, 0);
        lcd.print(" *");
      }
      curCount++;
      if (curCount > 13) curCount = 0;
    }
    if (millis() > nextRead1)
    {

  
      //считываем данные с термопар
      nextRead1 = millis() + SENSOR_SAMPLING_TIME;

      sprintf(buf, "$%d %d;", int(Input_f2), int(Input_f1)); //график ПК

      lcd.setCursor(16, 2);
      lcd.print("    ");
      lcd.setCursor(16, 2);
      if (isnan(Input1)) {
        lcd.print("Er");
      } else {
        lcd.print(tc1);
      }
      lcd.setCursor(16, 3);
      lcd.print("    ");
      lcd.setCursor(16, 3);
      if (isnan(Input2)) {
        lcd.print("Er");
      } else {
        lcd.print(tc2);
      }
    }

    if (millis() > nextRead3) {
      Output1 = Pid1(Input1, Setpoint1, kp1, ki1, kd1);
      Output2 = Pid2(Input2, Setpoint2, kp2, ki2, kd2);
      nextRead3 = millis() + PID_SAMPLING_TIME;
    }
    if (!TopStart) Output1 = 0;
  }
  else
  {
    digitalWrite(RelayPin1, LOW);
    digitalWrite(RelayPin2, LOW);
  }
}

void Dimming()
{
  if (reflowStatus == REFLOW_STATUS_ON)
  {
    OutPWR_TOP();
    OutPWR_BOTTOM();
  }
  if (Secs >= 100)
  {
    Serial.println(buf);
    Secs = 0;
  } else Secs++;
}

void OutPWR_TOP()
{
  //reg1 = round(Output1 * (pwr_TOP * 0.01)) + er1; //pwr- задание выходной мощности в %, er- ошибка округления
reg1 = Output1 + er1; //pwr- задание выходной мощности в %, er- ошибка округления

  if (reg1 < 50)
  {
    out1 = LOW;

    er1 = reg1 ; //reg- переменная для расчетов
  }
  else
  {
    out1 = HIGH;

    er1 = reg1 - 100;
  }
  digitalWrite(RelayPin1, out1); //пин через который осуществляется дискретное управление
}

void OutPWR_BOTTOM()
{
  //reg2 = round(Output2 * (pwr_BOTTOM * 0.01)) + er2; //pwr- задание выходной мощности в %, er- ошибка округления
reg2 = Output2 + er2; //pwr- задание выходной мощности в %, er- ошибка округления
  if (reg2 < 50)
  {
    out2 = LOW;

    er2 = reg2 ; //reg- переменная для расчетов
  }
  else
  {
    out2 = HIGH;

    er2 = reg2 - 100;
  }

  digitalWrite(RelayPin2, out2); //пин через который осуществляется дискретное управление
  //i++;
}

float filter1(float val_1) {  //функция фильтрации показений "Верхней" термопары
  Pc1 = P1 + varProcess1;
  G1 = Pc1 / (Pc1 + varTerm1);
  P1 = (1 - G1) * Pc1;
  Xp1 = Xe1;
  Zp1 = Xp1;
  Xe1 = G1 * (val_1 - Zp1) + Xp1; //"фильтрованное" значение
  return (Xe1);
}

float filter2(float val_2) {  //функция фильтрации показений "Нижней" термопары
  Pc2 = P2 + varProcess2;
  G2 = Pc2 / (Pc2 + varTerm2);
  P2 = (1 - G2) * Pc2;
  Xp2 = Xe2;
  Zp2 = Xp2;
  Xe2 = G2 * (val_2 - Zp2) + Xp2; //"фильтрованное" значение
  return (Xe2);
}

byte Pid1(double temp, byte ust, byte kP, byte kI, byte kd)
{
  byte out = 0;
  static float ed = 0;
  float e, p;
  float d;
  e = (ust - temp); //ошибка регулирования
  p =  (kP * e); //П составляющая
  integra = (integra < i_min) ? i_min : (integra > i_max) ? i_max : integra + (kI * e) / 100; //И составляющая
  d = kd * (e - ed) * 10; //Д составляющая
  ed = e;
  out = (p + integra + d < 0) ? 0 : (p + integra + d > pwr_TOP) ? 100 : p + integra + d;
  return out;
}

byte Pid2(double temp, byte ust, byte kP, byte kI, byte kd)
{
  byte out = 0;
  static float ed = 0;
  float e, p;
  float d;
  e = (ust - temp); //ошибка регулирования
  p =  (kP * e); //П составляющая
  integra2 = (integra2 < i_min) ? i_min : (integra2 > i_max) ? i_max : integra2 + (kI * e) / 100; //И составляющая
  d = kd * (e - ed) * 10; //Д составляющая
  ed = e;
  out = (p + integra2 + d < 0) ? 0 : (p + integra2 + d > pwr_BOTTOM) ? 100 : p + integra2 + d;
  return out;
}




